/***********************************************************************/
/*                                                                     */
/* RIGI C Parser --- Copyright (C) 1996-1999                           */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Maintainer: Johannes Martin (jmartin@csr.csc.uvic.ca)               */
/*                                                                     */
/***********************************************************************/
#include <stdio.h>
#include <string.h>
#include <assert.h>

#include "String.h"

void String::init(const char* s, int inc) {
  increment = inc;
  len = s ? strlen(s) : 0;
  allocation = ((len / increment) + 1) * increment;
  chars = new char[allocation];
  assert(chars);
  if (s)
    strcpy(chars, s);
  else
    chars[0] = '\0';
}

void String::alloc(int minLen) {
  int newAlloc = ((minLen / increment) + 1) * increment;
  if (newAlloc > allocation) {
    char *temp = chars;
    allocation = newAlloc;
    chars = new char[allocation];
    assert(chars);
    strcpy(chars, temp);
    delete[] temp;
  }
}

void String::realloc(int minLen) {
  int newAlloc = ((minLen / increment) + 1) * increment;
  if (newAlloc > allocation) {
    delete[] chars;
    allocation = newAlloc;
    chars = new char[allocation];
    assert(chars);
  }
}

String::String(int increment) {
  init(0, increment);
}

String::String(const char* s, int increment) {
  init(s, increment);
}

String::String(const unsigned long l, int increment) {
  char temp[64];
  sprintf(temp, "%lu", l);
  init(temp, increment);
}

String::String(const String& s) {
  init(s.chars, s.increment);
}

String::~String() {
  delete[] chars;
}

const String& String::append(const String& s) {
  alloc(len + s.len);
  strcpy(chars + len, s.chars);
  len += s.len;
  return *this;
}

const String& String::append(const String& s, int num) {
  alloc(len + num);
  strncpy(chars + len, s.chars, num);
  chars[len += num] = '\0';
  return *this;
}

const String& String::copy(const String& s) {
  realloc(len = s.len);
  strcpy(chars, s.chars);
  return *this;
}

const String& String::copy(const String& s, int num) {
  realloc(len = num);
  strncpy(chars, s.chars, num);
  chars[len] = '\0';
  return *this;
}

const String& String::operator=(const String& s) {
  return copy(s);
}

String::operator const char*() const {
  return chars;
}
