/***********************************************************************/
/*                                                                     */
/* RIGI C Parser --- Copyright (C) 1996-1999                           */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Maintainer: Johannes Martin (jmartin@csr.csc.uvic.ca)               */
/*                                                                     */
/***********************************************************************/
// $Author: jmartin $
// $Date: 2000/02/07 21:35:37 $
// $Source: /home/rigi/proj/cvs/Rigi/rigiparse/avl.cc,v $
// $Locker:  $
// $State: Exp $
//
// ------------------------------------------------------------------------

// $Author: jmartin $
// $Date: 2000/02/07 21:35:37 $
// $Source: /home/rigi/proj/cvs/Rigi/rigiparse/avl.cc,v $
// $Locker:  $
// $State: Exp $
//
// ------------------------------------------------------------------------

//
//	avl.c: AVL treesets
//
#include	<stdio.h>
#include	<ctype.h>
#include	<string.h>

#include	"avl.h"

const int INVALID	= -1;

static const int EVEN	= 0;
static const int LEFT	= 1;
static const int RIGHT	= 2;

static const int MAXSETS = 16; // HACK!

typedef int (*AvlCompFunc)(const void*,const void*);	// comparison function type

struct avlrecord
{
    const char *name;
    int info;
    int lean;
    struct avlrecord *left;
    struct avlrecord *right;
};
typedef struct avlrecord Record;

/*
#define	set_lean(x,l)	(x)->info = (((x)->info>>2)<<2) | (l)
#define	get_lean(x)	((x)->info & 0x3 )
#define	set_info(x,i)	(x)->info = ((x)->info & 0x3) | ((i)<<2)
#define	get_info(x)	((x)->info >> 2 )
*/
#define	set_lean(x,l)	(x)->lean = (l)
#define	get_lean(x)	((x)->lean)
#define	set_info(x,i)	(x)->info = (i)
#define	get_info(x)	((x)->info)

static char	types[MAXSETS];
static Record*	trees[MAXSETS];
static Record*	newrecord;
static Record*	NewRecord();
static int	FreeRecord( Record* );
static const char* newstring( const char* );

static int newsetindex=1;

static int info;
static int returnvalue;
static const char *string;

static int currtype;
// 0->key-on-info
// 1->key-on-string
// 2->key-on-other (name field points to an unknown record type)

static AvlCompFunc compare = (AvlCompFunc)strcmp;
static int AVL = 1;

// either store a new record, or just find it

static int Store( Record **tree, int level )
{
    // static vars: string, info, currtype, compare

    Record *n = *tree;
    int j;

    if (n==NULL)
    {
	*tree = n = newrecord = NewRecord();
	set_info(n,info);
	n->name = newstring( string );

	returnvalue = INVALID;
	string = n->name;

	// this tree just got heavier
	return 1;
    }

    // comparison
    j = (currtype ? info - get_info(n) : (*compare)( string, n->name ));

    if (j==0) // found the appropriate record
    {
	if (info!=INVALID) set_info(n,info);
	returnvalue = get_info(n);
	string = n->name;
	return 0;
    }
    else if (j<0) // recur down the tree
    {
	j = Store( &(n->left), level+1 ) ? LEFT : EVEN;
    }
    else
    {
	j = Store( &(n->right), level+1 ) ? RIGHT : EVEN;
    }
    if (j==0 || AVL==0) return 0;


    if (j!=get_lean(n))
    {
	// not a pivot - just set the lean factor
	if (get_lean(n)!=EVEN) set_lean(n,EVEN); else set_lean(n,j);

	// no change to nodes above if this subtree became EVEN
	return get_lean(n) ? 1 : 0;
    }

    // a rotation is required
    if (j==RIGHT && get_lean(n->right)==RIGHT)
    {
	set_lean(n,EVEN); set_lean(n->right,EVEN);

	*tree = (n->right);
	n->right = (n->right)->left;
	(*tree)->left = n;
	return 0;
    }
    else if (j==LEFT && get_lean(n->left)==LEFT)
    {
	set_lean(n,EVEN); set_lean(n->left,EVEN);

	*tree = (n->left);
	n->left = (n->left)->right;
	(*tree)->right = n;
	return 0;
    }
    else if (j==RIGHT && get_lean(n->right)==LEFT)
    {
	(*tree) = (n->right)->left;
	(n->right)->left = (*tree)->right;
	(*tree)->right = n->right;
	n->right = (*tree)->left;
	(*tree)->left = n;

	set_lean(n,get_lean(*tree)!=RIGHT ? EVEN : LEFT );
	set_lean((*tree)->right, get_lean(*tree)!=LEFT ? EVEN : RIGHT );
	set_lean(*tree,EVEN);
	return 0;
    }
    else if (j==LEFT && get_lean(n->left)==RIGHT)
    {
	(*tree) = (n->left)->right;
	(n->left)->right = (*tree)->left;
	(*tree)->left = n->left;
	n->left = (*tree)->right;
	(*tree)->right = n;

	set_lean(n,get_lean(*tree)!=LEFT ? EVEN : RIGHT);
	set_lean((*tree)->left,get_lean(*tree)!=RIGHT ? EVEN : LEFT);
	set_lean(*tree,EVEN);
	return 0;
    }
    else fprintf(stderr,"error: inconsistent state in AVL tree-set!\n");

    return 0;
}

int GetInfo( Avlset set, const char *s )
{
    return SetInfo( set, s, INVALID );
}

const char *GetString( Avlset set, int i )
{
    info = i; string = NULL;
    currtype = types[set];

    Store( trees+set,0 );
    return string;
}

int SetInfo( Avlset set, const char *s, int i )
{
    string = s; info = i;
    currtype = types[set];

    Store( trees+set,0 );
    return returnvalue;
}

void SetCompare( AvlCompFunc f ) { compare = f; }

static int maxlevel;
static int numset;
static int unbalanced;

static int Print( Record **tree, int level )
{
    Record *n = *tree; int l,r;

    if (n==NULL) return 0;

    ++ numset;
    if (level>maxlevel) maxlevel = level;

    l = 1 + Print( &(n->left), level+1 );
    printf("%2d%c %4d \"%s\"\n", level,
	get_lean(n)==EVEN?'E':(get_lean(n)==LEFT?'L':'R'),
	get_info(n), n->name );
    r = 1 + Print( &(n->right), level+1 );

    if ((r-l)>1 || (r-l)<-1) unbalanced = 1;
    return l>r ? l : r;
}

int PrintSet( Avlset set )
{
    int d;

    numset = maxlevel = unbalanced = 0;

    d = Print( trees+set, 0 );
    if (unbalanced)
	fprintf( stderr, "error: AVL sets not working properly\n" );
    if (maxlevel!=d-1)
	fprintf( stderr, "error: PrintSet() not working (d=%d)!\n",d );
    printf( "(total=%d maxdepth=%d)\n\n",numset,maxlevel );
    return numset;
}

Avlset NewSet( int type )
{
    if (newsetindex>=MAXSETS)
    {
	fprintf( stderr,"avlset error: overflow!!\n" );
	return INVALID;
    }
    ++newsetindex;
    types[newsetindex] = type;
    return newsetindex;
}

// MEMORY ALLOCATION

static Record *rlist = NULL;

static Record *NewRecord()
{
    static int maxindex = 64;
    static int index = maxindex;
    static const int maxblock = 512;
    static Record *array = NULL;

    Record *r;

    if (rlist) {
	r = rlist; rlist = r->right;
    } else if (index<maxindex) {
	r = array + index++;
    } else {
	if ( maxindex < maxblock ) maxindex<<=1;
	array = new Record[maxindex];
	index = 0;
	r = array + index++;
    }
    r->name = NULL; r->left = r->right = NULL;
    r->info = r->lean = 0;
    return r;
}

static int FreeRecord( Record *r )
{
    r->right = rlist; rlist = r;
    return 1;
}

static int FreeRecords( Record **tree )
{
    Record *n = *tree;
    if (n==NULL) return 0;
    return FreeRecords( &(n->left) )
    + FreeRecords( &(n->right) )
    + FreeRecord(n);
}

static const char *newstring( register const char *s )
{
    static int SIZE = 512;
    static char* index = NULL;
    static char* maxindex = NULL;

    register int len = strlen(s) + 1;
    register char *i = index;

    if (i+len>maxindex)
    {
	i = new char[SIZE];
	maxindex = i + SIZE;
    }
    if (*s) do { *i++ = *s++; } while (*s);
    *i++ = 0;
    index = i;
    return i-len;
}

int FreeSet( Avlset set )
{
    return FreeRecords( trees+set );
}

int AvlSet( int i ) { return AVL = i; }

int AvlMin( int d )
{
    if (d==0) return 1;
    else if (d==1) return 2;
    else return 1 + AvlMin(d-1) + AvlMin(d-2);
}

