/***********************************************************************/
/*                                                                     */
/* RIGI C Parser --- Copyright (C) 1996-1999                           */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Maintainer: Johannes Martin (jmartin@csr.csc.uvic.ca)               */
/*                                                                     */
/***********************************************************************/
// $Author: jmartin $
// $Date: 2000/08/13 22:29:29 $
// $Source: /home/rigi/proj/cvs/Rigi/rigiparse/cinit.cc,v $
// $Locker:  $
// $State: Exp $
//
// ------------------------------------------------------------------------


#include <stdio.h>
#include <string.h>
#include "globals.h"
#include "clex.h"
#include "cinit.h"
#include "cparse.h"		// for the token constants
#include "cl_symtbl.h"
#include "avl.h"		// to store new keywords (will be typenames)

Avlset newtokens = 0;

static int size;                // number of keywords

// Alphabetical order is imperative!

static struct Keyword {
	char	*name;
	int	token;
} Keywords [] = {
	"auto",		AUTO,
	"break",	BREAK,
	"case",		CASE,
	"char",		CHAR,
	"const",	CONST,
	"continue",	CONTINUE,
	"default",	DEFAULT,
	"do",		DO,
	"double",	DOUBLE,
	"else",		ELSE,
	"enum",		ENUM,
	"extern",	EXTERN,
	"float",	FLOAT,
	"for",		FOR,
	"goto",		GOTO,
	"if",		IF,
	"int",		INT,
	"long",		LONG,
	"register",	REGISTER,
	"return",	RETURN,
	"short",	SHORT,
	"signed",	SIGNED,
	"sizeof",	SIZEOF,
	"static",	STATIC,
	"struct",	STRUCT,
	"switch",	SWITCH,
	"typedef",	TYPEDEF,
	"typeof",       TYPEOF,
	"union",	UNION,
	"unsigned",	UNSIGNED,
	"void",		VOID,
	"volatile",	VOLATILE,
	"while",	WHILE,
};

void compiler_init()
{
    size = sizeof(Keywords)/sizeof(Keyword);
}

int findtoken( const char* name )
{
        register struct Keyword *r;
        register int low=0; register int high=size-1;
        register int i,mid;

	// First, see if the word is in the keyword table

        do {
                mid = (low+high)>>1;
                r = Keywords + mid;
                i = strcmp(name,r->name);
                if (i==0) return r->token;
                else if (i>0) low = mid+1;
                else high = mid-1;

        } while (low<=high);

	// Check to see if this keyword is a "new" token

	i = GetInfo( newtokens, name );
	if (i>=0) return i;
	
        return IDENTIFIER;
}

int newtype( const char *name, int value )
{
    static int num_undefined = 0;

    if (!name) {
      yyerror("internal error: newtype - NULL name");
      return 0;
    }
    if (!newtokens) newtokens = NewSet(0);
    SetInfo( newtokens, name, value );

    if (symdebug)
        fprintf(stderr,"newtype:  Creating type %s, token = %d\n",
                name, value);
    return 1;
}

void ParseInfo::MakeName() {
  static unsigned short unnamed = 0;

  if (name) delete[] name;

  int len = strlen(file) + 1 + 7 + 1 + 5 + 1;

  name = new char[len];

  sprintf(name, "%s.unnamed.%d", file, unnamed++);
}
