/***********************************************************************/
/*                                                                     */
/* RIGI C Parser --- Copyright (C) 1996-1999                           */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Maintainer: Johannes Martin (jmartin@csr.csc.uvic.ca)               */
/*                                                                     */
/***********************************************************************/
// $Author: jmartin $
// $Date: 1999/06/07 15:46:27 $
// $Source: /home/rigi/proj/cvs/Rigi/rigiparse/cl_avlset.h,v $
// $Locker:  $
// $State: Exp $
//
// ------------------------------------------------------------------------

#ifndef __AVLSET_H__
#define __AVLSET_H__

class AvlRecord;
class AvlPos;

typedef void Info;
typedef int (*AvlFunc)( void*,Info* );

// Iterator
#define foreach( member, set) \
{ \
    AvlPos *pos; Info *member; \
    for ( member=set->First(pos); member; member=set->Next(pos) ) {
#define endfor }}


class AvlSet
{
    private:

        int         size;          // number of elements (and flags)
        AvlRecord*  contents;      // pointer to root of data structure

    public:

        AvlSet();
        AvlSet( int ); // permit duplicate keys
        ~AvlSet();

        int	Size()		{ return size>>8; }

        Info*	StoreInfo ( void* key,Info*,AvlFunc=NULL );
        Info*	RemoveInfo( void* key,Info*,AvlFunc=NULL );
        Info*	FindInfo  ( void* key,AvlFunc=NULL );

        void	Print( void(*)(Info*) );
        void	Count();
        int	Empty();

        Info*	First();
        Info*	Last();

        Info*	First( AvlPos*& );
        Info*	First( AvlPos*&, void* key, AvlFunc=NULL );
        Info*	Next( AvlPos*& );
        Info*	LastInfo( AvlPos*& );
        void	Delete( AvlPos* );

        Info*	Iterate( int(*)(Info*) );
};

// Description:
//
//	Generic log(n) access AVL tree sets, permitting multiple
//	members with the same key (if argument to constructor is 1).
//	Default "type" of set is "no members with the same key",
//	but actual stored Info records NEVER occur more than once.
//
//	StoreInfo() stores the given info by the given key. Previously
//	    held info is returned.
//	RemoveInfo() finds and removes the given info. If info parameter
//	    is NULL, it removes the first info matching the key.
//	FindInfo() returns the first found info given the key.
//
//	First() and Last() return the first/last members of the set,
//	    but iteration is not set up.
//	First(AvlPos*) and Next(AvlPos*) iterate through the set.
//	LastInfo(AvlPos*) give the info returned by last First/Next call.
//	Delete(AvlPos*) frees mem assoc. with iteration iff iteration
//	    is terminated early (First/Next NOT yet returned NULL).
//	Iterate() goes through members in order, until f returns 0,
//	    at which point that info is returned.
//
//	Set manipulation (add,remove,empty) must not occur while iterating!

#endif
