/***********************************************************************/
/*                                                                     */
/* RIGI C Parser --- Copyright (C) 1996-1999                           */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Maintainer: Johannes Martin (jmartin@csr.csc.uvic.ca)               */
/*                                                                     */
/***********************************************************************/
// $Author: jmartin $
// $Date: 1999/06/07 15:46:27 $
// $Source: /home/rigi/proj/cvs/Rigi/rigiparse/cl_fldtbl.h,v $
// $Locker:  $
// $State: Exp $
//
// ------------------------------------------------------------------------

#ifndef __CL_FLDTBL_H__
#define __CL_FLDTBL_H__

#include <stdlib.h>
#include <string.h>
#include "cl_avlset.h"

#define INVALID_ATTR       (-1)

class FieldKey {

  public:
    FieldKey(const char* s, const char* f)   { InitKey(s, f); }
    FieldKey(const FieldKey& f) { InitKey(f.structure, f.field); }
    ~FieldKey()                 { delete[] structure; delete[] field; }
    const char* Field()         { return field; }
    const char* Structure()     { return structure; }
    FieldKey& operator=(const FieldKey& f) {
        delete[] structure;
        delete[] field;
        InitKey(f.structure, f.field);
        return *this;
    }
    int operator==(const FieldKey& f) const {
        int primary = strcmp(f.structure, structure);
        if (primary == 0) {
            primary = strcmp(f.field, field);
        }
        return primary;
    }

  private:
    char *structure, *field;
    void InitKey(const char *s, const char *f) {
        structure = new char[strlen(s)+1];
        strcpy(structure, s);
        field = new char[strlen(f)+1];
        strcpy(field, f);
    }
};

class FldTbl : private AvlSet
{

    class Field
    {

      // A field has a key (the name of the structure and the name of the
      // field) and a value (the fields type)

      public:

        FieldKey *key;
	char *type;

	Field( const char*s, const char*f, const char*t ) {
            key = new FieldKey(s, f);
            type = new char[strlen(t)+1];
            strcpy(type, t);
        }
	~Field() {
            delete key;
            delete[] type;
        }
    };

    static int ff( Info *attr ) { delete (Field*)attr; return 0; }
    static int cmp( void *a, Field *fld )
    {
        int rval = (*(fld->key) == *((FieldKey*)a));
        return rval;
    }
    static void p( Field *attr )
    {
        printf("<%s,%s> %s",attr->key->Structure(),
                            attr->key->Field(),
                            attr->type);
    }

    public:

	FldTbl()  { }
	~FldTbl() { Iterate((int(*)(Info*))ff); }

	int Size()		{ return AvlSet::Size(); }
	int Empty()		{ return AvlSet::Empty(); }

	FieldKey* First( AvlPos*& P )
	{
	    Field *attr = (Field*)AvlSet::First(P);
	    return attr ? attr->key : NULL;
	}
	FieldKey* Next( AvlPos*& P )
	{
	    Field *attr = (Field*)AvlSet::Next(P);
	    return attr ? attr->key : NULL;
	}

	int SetField( const char *structure, const char* field, const char *type )
	{
            int rval = 0;
	    Field *attr = new Field(structure, field, type);
	    attr = (Field*)
		StoreInfo( (void*)(attr->key), (Info*)attr, (AvlFunc)cmp );
	    if (attr) { delete attr; rval = 1; } else rval = 0;
	    return rval;
	}
	const char* GetField( const char *structure, const char* field )
	{
            FieldKey key(structure, field);
	    Field *attr = (Field*)
		FindInfo( (void*)(&key), (AvlFunc)cmp );
	    return attr ? attr->type : NULL ;
	}
	int RemoveField( const char *structure, const char* field )
        {
            int rval = 0;
            FieldKey key(structure, field);
	    Field *attr = (Field*)
		RemoveInfo( (void*)(&key), NULL, (AvlFunc)cmp );
	    if (attr) { delete attr; rval = 1; }
	    return rval;
        }

	void Print() { AvlSet::Print( (void(*)(void*))p ); }
};

#endif
