/***********************************************************************/
/*                                                                     */
/* RIGI C Parser --- Copyright (C) 1996-1999                           */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Maintainer: Johannes Martin (jmartin@csr.csc.uvic.ca)               */
/*                                                                     */
/***********************************************************************/
// $Author: jmartin $
// $Date: 1999/06/07 15:46:27 $
// $Source: /home/rigi/proj/cvs/Rigi/rigiparse/cl_symtbl.h,v $
// $Locker:  $
// $State: Exp $
//
// ------------------------------------------------------------------------

#ifndef __CL_SYMTBL_H__
#define __CL_SYMTBL_H__

#include <stdlib.h>
#include <string.h>
#include "cl_avlset.h"
#include "cinit.h"

#define INVALID_ATTR       (-1)

class SymTbl : private AvlSet {
  class Symbol {
    // A symbol has a key (the symbol name), a value (the symbols type)
    // and a fully qualified name (the symbol's name + scope)

  public:
    ParseInfo info;
    Symbol(const ParseInfo& info) {
      this->info = info;
    }
  };

  static int ff(Info *attr) {
    delete (Symbol*)attr;
    return 0;
  }

  static int cmp(const char *a, Symbol *attr) {
    return strcmp(a,attr->info.Name());
  }

  static void p(Symbol *attr) {
    printf("%s (%s) %s",attr->info.Name(), attr->info.QualifiedName(),
	   attr->info.Type());
  }
public:

  ~SymTbl() {
    Iterate((int(*)(Info*))ff);
  }

  int Empty() {
    return AvlSet::Empty();
  }

  int SetSymbol(const ParseInfo& info ) {
    Symbol *attr = new Symbol(info);
    attr = (Symbol*) StoreInfo((void*)info.Name(), (Info*)attr, (AvlFunc)cmp);
    if (attr) {
      delete attr;
      return 1;
    }
    return 0;
  }

  const ParseInfo* GetSymbol(const char *key) {
    Symbol *attr = (Symbol*) FindInfo((void*)key, (AvlFunc)cmp);
    return attr ? &attr->info : NULL;
  }

  void Print() {
    AvlSet::Print( (void(*)(void*))p );
  }
};

#endif
