/***********************************************************************/
/*                                                                     */
/* RIGI C Parser --- Copyright (C) 1996-1999                           */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Maintainer: Johannes Martin (jmartin@csr.csc.uvic.ca)               */
/*                                                                     */
/***********************************************************************/
%{

// $Author: jmartin $
// $Date: 2000/08/13 22:29:29 $
// $Source: /home/rigi/proj/cvs/Rigi/rigiparse/clex.l,v $
// $Locker:  $
// $State: Exp $

//------------------------------------------------------------------------

// ------------------------------------------------------------------------
// clex.l: [SRT 08Feb93]
/*----------------------------------------------------------------------*/
/*									*/
/*		LEX generated C lexical analyser			*/
/*									*/
/*----------------------------------------------------------------------*/
//
#include <stream.h>
#include <string.h>
#include "globals.h"
#include "clex.h"
#include "cinit.h"
#include "cparse.h"
#include "parse.h"
#include "symtbl.h"

#define LEX_RETURN(x)   count();return checkt(x)

// Supplied routines
static int ident();
static int checkt(int);
static void strings(int);
static void count();
static void comment(char*);
static void directive(char*);
static void asmStatement();

int lex_column = 1;
int lex_line = 1;
char *lex_file = NULL;

#define		yywrap()	(1)

%}

D			[0-9]
NUM			({D}+|({D}+"."{D}*)|("."{D}+))([eE][-+]?{D}+)?[uU]?[lL]?[lL]?[fF]?
HEX			0[xX][0-9a-fA-F]+[uU]?[lL]?[lL]?[fF]?
ID			[A-Za-z_][A-Za-z0-9_]*
ASG			([-+*/%&^|]?"=")|"<<="|">>="

%%

"#".*"\n"		{ count(); lex_line++; directive(yytext); }
"const"			{ count(); }
"volatile"		{ count(); }
"inline"                { count(); }
"asm"                   { asmStatement(); LEX_RETURN (ASM); }
{ID}                    { int t = ident(); count(); return checkt(t); }
{NUM}			{ yylval.info = ParseInfo(NULL, "int", NULL, lex_file, lex_line, lex_column); LEX_RETURN( CONSTANT ); }
{HEX}			{ yylval.info = ParseInfo(NULL, "int", NULL, lex_file, lex_line, lex_column); LEX_RETURN( CONSTANT ); }
{ASG}			{ LEX_RETURN( Asgn_Op ); }
"\""			{ yylval.info = ParseInfo(NULL, "char", NULL, lex_file, lex_line, lex_column); strings ('"'); LEX_RETURN(STRING_CONSTANT); }
"'"			{ yylval.info = ParseInfo(NULL, "char", NULL, lex_file, lex_line, lex_column); strings ('\''); LEX_RETURN(STRING_CONSTANT); }
"<<"			{ LEX_RETURN( Shift ); }
">>"			{ LEX_RETURN( Shift ); }
"&&"			{ LEX_RETURN( And ); }
"||"			{ LEX_RETURN( Or ); }
"->"			{ LEX_RETURN( Arrow ); }
"<"|"<="|">="|">"	{ LEX_RETURN( Rel_Op ); }
"=="|"!="		{ LEX_RETURN( Eq_Op ); }
"++"|"--"		{ LEX_RETURN( IncDec ); }
"..."			{ LEX_RETURN( ELLIPSIS ); }

"/*"			{ comment ("*/"); }
"//"			{ comment ("\n"); }

"\n"                    { lex_column = 1 ; lex_line++ ; }
[ \t\v\f]		{ count(); }
.			{ LEX_RETURN( yytext[0] ); }

%%

static int checkt( int token )
{
    if(debug && lex_line>=debugline)
        fprintf(stdout,"Returning token %d\t%d\t%s\n",token,lex_column,yytext);
    return token;
}

static int ident()
{
    // Handle an Identifier or Reserved Word
    const ParseInfo *temp = findsym(yytext);
    if (temp != NULL)
      yylval.info = ParseInfo(yytext, temp->Type(), temp->QualifiedName(),
			      lex_file, lex_line, lex_column);
    else
      yylval.info = ParseInfo(yytext, NULL, yytext,
			      lex_file, lex_line, lex_column);
//    fprintf(stderr,"ident: yytext = %s, info = %s,%s\n", yytext,
//            yylval.info->Name(), yylval.info->Type());
    return findtoken(yytext);
}

static void count()
{
    int i;

    if (lex_column == 1) fflush(yyout);

    for ( i=0; yytext[i] != '\0'; i++ )

	if (yytext[i] == '\n') {
		lex_column = 1;
	}
	else if (yytext[i] == '\t') {
		lex_column += 8 - ((lex_column - 1) % 8);
	}
	else {
		lex_column++;
	}
}


static void comment( char *s )
{
    //  Swallow the rest of a comment, up to EOF or s
    register int c=0, i=0;
    if (!s) fprintf(stderr,"problem in comment()\n");

    for (;;)
    {
        c = yyinput();
        if (c==EOF) break;
        if (c=='\n') { lex_line++; lex_column=1; } else lex_column++;
        if (c==s[i]) { i++; if (s[i]=='\0') break; }
            else i = (c==s[0]) ? 1 : 0; // HACK! Not general
    }
}

static char* fullName( char* fileName )
{
  char tempwd[1024];
  getcwd(tempwd, sizeof(tempwd));
  char *dirsep;
  if (((dirsep = strchr(tempwd, '/')) == NULL) &&
      ((dirsep = strchr(tempwd, '\\')) == NULL) &&
      ((dirsep = strchr(tempwd, ':')) == NULL))
    return strdup(fileName);

  char newwd[1024];
  char *pathEnd;
  if ((pathEnd = strrchr(fileName, *dirsep)) == NULL)
    strcpy(newwd, tempwd);
  else {
    *pathEnd = '\0';
    chdir(fileName);
    getcwd(newwd, sizeof(newwd));
    *pathEnd = *dirsep;
  }
  char *result = (char*)malloc(strlen(newwd) + strlen(fileName) + 2);
  strcpy(result, newwd);
  strncat(result, dirsep, 1);
  if (pathEnd == NULL)
    strcat(result, fileName);
  else {
    strcat(result, pathEnd + 1);
    chdir(tempwd);
  }
  if (pathPrefix && (strncmp(result, pathPrefix, strlen(pathPrefix)) == 0))
    strcpy(result, result + strlen(pathPrefix));
  return result;
}

static void directive( char* str )
{
    static int lastline = 0;
    char namebuff[256];
    int x, line=0;
    if (!strncmp( str+1,"pragma",6 )) return;
    if (!strncmp( str+1,"ident",5 )) return;

    // see if we can find a line number
    x = sscanf( str+1,"%d %255s",&line,namebuff );
    if (x < 1 || x > 2 || line==0)
    {
	x = sscanf( str+1, "line%*[^0-9]%d %255s", &line, namebuff);
	if (x < 1 || x > 2 || line== 0) {
            fprintf( stderr,"unknown directive: file %s, line %d: %s",
	             lex_file,lex_line-1,str );
	    return;
	}
    }

    {
        int bufflen, i;
        if (ignoreline) return;
        //lex_lines += lex_line-lastline;

        /* If the string is quoted, strip off the quotes */

	if (x > 1) {
            bufflen = strlen(namebuff);
            if (namebuff[0] == '"' && namebuff[bufflen-1] == '"') {
                for(i=0 ; i<bufflen-2 ; i++)
                    namebuff[i] = namebuff[i+1];
                namebuff[bufflen-2] = 0;
            }
	}
	free(lex_file);
	lex_file = fullPath ? fullName(namebuff) : strdup(namebuff);
        lex_line = lastline = line;
	newfile(lex_file);
    }
}

static void processAsmStatement(int c, int& parCount) {
  switch (c = yyinput()) {
    case '(' : parCount++; lex_column++; break;
    case ')' : parCount--; lex_column++; break;
    case '"' : ;
    case '\'': strings(c); break;
    case '\n': lex_line++; lex_column=1; break;
    default: lex_column++;
  }
}

static void asmStatement() {
  int parCount = 0;
  int c = ' '; 

  while ((c != EOF) && (parCount == 0))
    processAsmStatement(c, parCount);

  while ((c != EOF) && (parCount > 0)) 
    processAsmStatement(c, parCount);
}

static void strings (int term)
{
	/*  A string terminating with 'term'  */

	register int c;
        int i = 1;

	while(c = yyinput ())	{
                yytext[i++] = c;
		if (c == term) {
                        yytext[i] = '\0';
			break;
                } else if (c == '\\') {
			yytext[i++] = yyinput ();
                }
	}
}

void lex_init( char *filename )
{
        lex_column      = 1 ;
        lex_line        = 1 ;
	lex_file	= fullPath ? fullName(filename) : strdup(filename);
        newfile(lex_file);
}

void yyerror(char *s)
{
    fprintf(stderr,"syntax error, ");
    fprintf(stderr,"line %d(%d) in file %s:",lex_line,lex_column,lex_file);
    fprintf(stderr," %s\n",s);
    if (syntax_stop) {
      char c;
      fprintf(stderr, "-->%s", yytext);
      do {
	c = yyinput();
	fprintf(stderr, "%c", c);
      } while ((c != EOF) && (c != '\n'));
      if (c == EOF)
        fprintf(stderr, "%c", '\n');
      fflush(stdout);
      fflush(stderr);
      exit(1);
    }
    fflush(stderr);
}

