/***********************************************************************/
/*                                                                     */
/* RIGI C Parser --- Copyright (C) 1996-1999                           */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Maintainer: Johannes Martin (jmartin@csr.csc.uvic.ca)               */
/*                                                                     */
/***********************************************************************/
// $Author: jmartin $
// $Date: 2000/08/16 21:56:58 $
// $Source: /home/rigi/proj/cvs/Rigi/rigiparse/main.cc,v $
// $Locker:  $
// $State: Exp $
//
// ------------------------------------------------------------------------


// MJW: C parse - converts to RSF triples

#include <stdio.h>
#include <iostream.h>
#include <string.h>		// for rindex
#include <stdlib.h>
#include <signal.h>

#include "globals.h"

static char *filename = NULL;

// interface to parsing stuff
extern FILE *yyin;
extern void lex_init( char* );
extern void compiler_init();
extern "C" int yyparse();
extern "C" void yyerror(char* s);

void Help(const char* execName)
{
    cerr << PRODUCT " - " VERSION "\n";
    cerr << "usage: " << execName << "[options] [<inputFile>]\n";
    cerr << "\t-a\t\tgenerate 'lineno' and 'file' attribute\n";
    cerr << "\t-A\t\tgenerate 'File' nodes and 'isInFile' arcs\n";
    cerr << "\t-f[prefix]\temit full path name for all file names in RSF output\n";
    cerr << "\t\t\tif prefix is specified, remove the prefix from all\n";
    cerr << "\t\t\tfilenames that start with this prefix\n";
    cerr << "\t-h\t\tprint this help\n";
    cerr << "\t-o file\t\tfile for output\n";
    cerr << "\t-q\t\tissue no relations in output\n";
    cerr << "\t-s\t\tstop at first syntax error\n";
    cerr << "\t-x[line]\tlex debug (start at line)\n";
    cerr << "\t-4\t\temit file name and line number as element 4 of tuple\n";
}

static int Parse() {
    int r;

    if (!filename) {
      yyin = stdin;
      filename = "<stdin>";
    } else {
      if ((yyin = fopen( filename,"r" )) == NULL) {
	cerr << PRODUCT ": error - cannot open input file ";
	cerr << filename << "\n";
        exit(1);
      }
      cerr << PRODUCT ": starting to parse file \"" << filename << "\"\n";
    }

    lex_init( filename );
    compiler_init();

    r = yyparse();

    if (r) 
      cerr << PRODUCT ": parsing failed\n";
    return r;
}

void SignalHandler(int signal) {
  fprintf(stderr, "Caught signal %d.\n\n", signal);
  syntax_stop = 1;
  yyerror("Abnormal program termination");
}

void main( int argc, char **argv ) {
    signal(SIGSEGV, SignalHandler);

    FILE *excludeFile = NULL;
    const char* execName = argv[0];

    // process arguments

    while (--argc) {

        if (**++argv=='-') {
            switch (*(*argv+1)) {
            case 'd':
	            symdebug = 1;
                break;
	    case 'f':
	        fullPath = 1;
		if (*(*argv + 2))
		  pathPrefix = *argv + 2;
		break;
            case 'l':
                ignoreline = 1;
                break;
            case 'o':
                argc--;
                if ((outfile = fopen(*++argv, "w")) == NULL) {
                    cerr << PRODUCT ": error - cannot open output file ";
                    cerr << argv << "\n";
                    exit(1);
                }
                break;
	    case 'h':
                Help(execName);
                exit(0);
            case 'q':
                quiet = 1;
                break;
	    case 's':
	        syntax_stop = 1;
                break;
            case 'a':
                fileline = 1;
                break;
	    case 'A':
	        fileNode = 1;
		break;
            case 'w':
                dowarn = 1;
                break;
            case 'x':
                debug = 1;
                debugline = atoi(*argv+2);
                break;
            case '4':
                write4 = 1;
                break;
            default:
                cerr << PRODUCT ": error - unrecognized command line argument ";
                cerr << *argv << "\n";
                cerr << "Use -h for help.\n";
                exit(1);
            }
	    } else {
        	filename = *argv;
        }
    }

    exit(Parse());
}

